#include "include.h"
#include <math.h>

void drawPlane(GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat);
void initDisplayLists();
void drawText(const CHAR *, GLfloat, GLfloat);
void renderActors();
void getPhysicsResults();
void startPhysics();
NxActor *CreateZPlane();
void updateCamera(POINT);
void crossProduct(GLfloat *, GLfloat *, GLfloat *);
void rotateVector(GLfloat, GLfloat *, GLfloat, GLfloat, GLfloat);
void setupLights();
void setupTextures();
void setupLowLevelShaders();
void setupHighLevelShaders();
CHAR *getShader(CHAR *);

GLuint	myProgram;


Frame camera = {0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 1.0f};

const char *sTexFiles[6] = {	"media/sphere.tga",  "media/sphere_normal.tga", "media/sphere_height.tga",
					"media/cube.tga", "media/cube_normal.tga", "media/cube_height.tga" };
int	iTexFiles = 6;

void drawScene(HWND hwnd, HDC hdc, POINT mousePos)
{
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	updateCamera(mousePos);

	glUseProgram(myProgram);
	glColor3f(1.0f, 1.0f, 1.0f);
	glActiveTexture(GL_TEXTURE0);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, texObject[0]);
	glActiveTexture(GL_TEXTURE1);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, texObject[1]);
	{
		int ret;
		ret = glGetUniformLocation(myProgram, "texture1"); 
		glUniform1i(ret, 1);
		ret = glGetUniformLocation(myProgram, "texture2"); 
		glUniform1i(ret, 2);
		ret = glGetUniformLocation(myProgram, "vertexTangent"); 
		glUniform3f(ret, 0.0f, 1.0f, 0.0f);
	}
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(-5.0f, -5.0f, -20.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(5.0f, -5.0f, -20.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(5.0f, 5.0f, -20.0f);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(-5.0f, 5.0f, -20.0f);
	glEnd();
	glUseProgram(0);

	if (gScene)
	{
		getPhysicsResults();
		startPhysics();
	}
	glEnable(GL_LIGHTING);
	glActiveTexture(GL_TEXTURE1);
	glDisable(GL_TEXTURE_2D);
	renderActors();
	glActiveTexture(GL_TEXTURE0);
	glDisable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE1);
	glDisable(GL_TEXTURE_2D);

	glColor3f(1.0f, 1.0f, 1.0f);
	drawPlane(0.0f, 1.0f, 0.0f, -1.0f, 20.0f, 1.0f);
	glDisable(GL_LIGHTING);
	{
		CHAR	buffer[80];

		glColor3f(1.0f, 1.0f, 1.0f);
		wsprintf(buffer, "%s", glGetString(GL_VENDOR));
		drawText(buffer, 0.0f, 0.15f);
		wsprintf(buffer, "%s", glGetString(GL_RENDERER));
		drawText(buffer, 0.0f, 0.1f);
		wsprintf(buffer, "OpenGL Version: %s", glGetString(GL_VERSION));
		drawText(buffer, 0.0f, 0.05f);
	}
	SwapBuffers(hdc);
}

void initGL(void)
{
	GLfloat		fLargest;

	glClearColor(0.25f, 0.25f, 0.25f, 1.0f);
	setupLights();
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	setupTextures();
	glEnable(GL_TEXTURE_2D);
//	glEnable(GL_MULTISAMPLE_ARB);
//	glHint(GL_MULTISAMPLE_FILTER_HINT_NV, GL_NICEST);
//	glGetFloatv(GL_TEXTURE_MAX_ANISOTROPY_EXT, &fLargest);
//	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, fLargest);

//	setupLowLevelShaders();
//	glEnable(GL_VERTEX_PROGRAM_ARB);
//	glEnable(GL_FRAGMENT_PROGRAM_ARB);

	setupHighLevelShaders();

	initDisplayLists();
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);


}

void setupLights()
{
	GLfloat		lightAmbient[]	= {0.25f, 0.25f, 0.25f, 1.0f},
			lightDiffuse[]	= {0.5f, 0.5f, 0.5f, 1.0f},
			lightSpecular[]	= {0.8f, 0.8f, 1.0f, 0.5f},
			lightPos[]	= {1.5f, 1.5f, 1.5f, 0.0f};

	glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lightSpecular);
	glLightfv(GL_LIGHT0, GL_POSITION, lightPos);
	glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SEPARATE_SPECULAR_COLOR);

	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	glMaterialfv(GL_FRONT, GL_SPECULAR, lightSpecular);
	glMateriali(GL_FRONT, GL_SHININESS, 16);
	glShadeModel(GL_SMOOTH);
}

void setupTextures()
{
	GLbyte		*pBytes;
	GLint		iWidth, iHeight, iComponents;
	GLenum		eFormat;
	int		i;

	glGenTextures(iTexFiles, texObject);
	for(i = 0; i < iTexFiles; i++)
	{
		glBindTexture(GL_TEXTURE_2D, texObject[i]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
		glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE);
		pBytes = gltLoadTGA(sTexFiles[i], &iWidth, &iHeight, &iComponents, &eFormat);
		glTexImage2D(GL_TEXTURE_2D, 0, iComponents, iWidth, iHeight, 0, eFormat, GL_UNSIGNED_BYTE, pBytes);
		free(pBytes);
	}
}

void setupLowLevelShaders()
{
	CHAR		*shader;
	GLint		errorPos, i;
	GLuint		pixelShader, vertexShader;
	GLfloat		fLargest;

	//low level shaders
	glGenProgramsARB(1, &vertexShader);
	glGenProgramsARB(1, &pixelShader);
	
	shader = getShader("shaders/asm/vertexShader.vp");
	if (shader)
	{
		glBindProgramARB(GL_VERTEX_PROGRAM_ARB, vertexShader);
		glProgramStringARB(GL_VERTEX_PROGRAM_ARB, GL_PROGRAM_FORMAT_ASCII_ARB, lstrlen(shader), shader);
		free(shader);
		glGetIntegerv(GL_PROGRAM_ERROR_POSITION_ARB, &errorPos);
		if (errorPos != -1)
		{
			CHAR *sError;

			sError = (char *)glGetString(GL_PROGRAM_ERROR_STRING_ARB);
			//log error, disable shader
		}
	}

	shader = getShader("shaders/asm/pixelShader.fp");
	if (shader)
	{
		glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, pixelShader);
		glProgramStringARB(GL_FRAGMENT_PROGRAM_ARB, GL_PROGRAM_FORMAT_ASCII_ARB, lstrlen(shader), shader);
		free(shader);
		glGetIntegerv(GL_PROGRAM_ERROR_POSITION_ARB, &errorPos);
		if (errorPos != -1)
		{
			CHAR *sError;

			sError = (char *)glGetString(GL_PROGRAM_ERROR_STRING_ARB);
			//log error, disable shader
		}
	}

}

void setupHighLevelShaders()
{
	GLuint		myVertexShader, myPixelShader;
	const GLchar	*myStringPtrs[1];
	GLchar		*sVertexShader, *sPixelShader;
	FILE		*fLog;
	GLint		success;

	fLog = fopen("infolog.txt", "w");
	
	sVertexShader = getShader("shaders/glsl/lighting.vs");
	if (sVertexShader == NULL)
	{
		fprintf(fLog, "unable to load vertexShader");
		fclose(fLog);
		return;
	} else {
		fprintf(fLog, "Loaded vertex shader\n");
	}

	myStringPtrs[0] = sVertexShader;
	myVertexShader = glCreateShader(GL_VERTEX_SHADER);
	glShaderSource(myVertexShader, 1, myStringPtrs, NULL);
	glCompileShader(myVertexShader);
	glGetShaderiv(myVertexShader, GL_COMPILE_STATUS, &success);
	if (!success)
	{
		GLint		infoSize;
		GLchar		*infoLog;

		glGetShaderiv(myVertexShader, GL_INFO_LOG_LENGTH, &infoSize);
		infoLog = (GLchar *)malloc(sizeof(GLchar) * infoSize);

		glGetShaderInfoLog(myVertexShader, infoSize, NULL, infoLog);
		fprintf(fLog, "Error compiling: %s", infoLog);
		fclose(fLog);
		return;
	}


	sPixelShader = getShader("shaders/glsl/normalmap.fs");
	if (sPixelShader == NULL)
	{
		fprintf(fLog, "unable to load pixelShader");
		fclose(fLog);
		return;
	} else {
		fprintf(fLog, "Loaded fragment shader\n");
	}
	myStringPtrs[0] = sPixelShader;
	myPixelShader = glCreateShader(GL_FRAGMENT_SHADER);
	glShaderSource(myPixelShader, 1, myStringPtrs, NULL);
	glCompileShader(myPixelShader);
	glGetShaderiv(myPixelShader, GL_COMPILE_STATUS, &success);
	if (!success)
	{
		GLint		infoSize;
		GLchar		*infoLog;

		glGetShaderiv(myPixelShader, GL_INFO_LOG_LENGTH, &infoSize);
		infoLog = (GLchar *)malloc(sizeof(GLchar) * infoSize);

		glGetShaderInfoLog(myPixelShader, infoSize, NULL, infoLog);
		fprintf(fLog, "Error compiling: %s", infoLog);
		fclose(fLog);
		return;
	}

	myProgram = glCreateProgram();
	glAttachShader(myProgram, myVertexShader);
	glAttachShader(myProgram, myPixelShader);
	glLinkProgram(myProgram);
	glGetProgramiv(myProgram, GL_LINK_STATUS, &success);
	if (!success)
	{
		GLint		infoSize;
		GLchar		*infoLog;

		glGetProgramiv(myProgram, GL_INFO_LOG_LENGTH, &infoSize);
		infoLog = (GLchar *)malloc(sizeof(GLchar) * infoSize);

		glGetProgramInfoLog(myProgram, infoSize, NULL, infoLog);
		fprintf(fLog, "Error linking: %s", infoLog);
		fclose(fLog);
		return;
	} else {
		fprintf(fLog, "Link successful\n");
	}
}

void initDisplayLists()
{
	GLUquadricObj *quadObj = gluNewQuadric();

	gluQuadricDrawStyle(quadObj, GLU_FILL);
	gluQuadricNormals(quadObj, GLU_SMOOTH); 
	gluQuadricOrientation(quadObj, GLU_OUTSIDE);
	gluQuadricTexture(quadObj, GL_TRUE);

	dispListSphere = glGenLists(1);
	glNewList(dispListSphere, GL_COMPILE);
		gluSphere (quadObj, 0.5f, 16, 16);
	glEndList();

	dispListCube = glGenLists(1);
	glNewList(dispListCube, GL_COMPILE);
		glPushMatrix();
		glTranslatef(-0.5f, -0.5f, -0.5f); // I just really like having the points positively based
		glBegin(GL_QUADS);
		glNormal3f(0.0f, 0.0f, 1.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(0.0f, 0.0f, 1.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(1.0f, 0.0f, 1.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(1.0f, 1.0f, 1.0f);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(0.0f, 1.0f, 1.0f);

		glNormal3f(0.0f, 1.0f, 0.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(0.0f, 1.0f, 0.0f);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(0.0f, 1.0f, 1.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(1.0f, 1.0f, 1.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(1.0f, 1.0f, 0.0f);


		glNormal3f(0.0f, -1.0f, 0.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(0.0f, 0.0f, 0.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(1.0f, 0.0f, 0.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(1.0f, 0.0f, 1.0f);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(0.0f, 0.0f, 1.0f);

		glNormal3f(0.0f, 0.0f, -1.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(0.0f, 0.0f, 0.0f);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(0.0f, 1.0f, 0.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(1.0f, 1.0f, 0.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(1.0f, 0.0f, 0.0f);

		glNormal3f(1.0f, 0.0f, 0.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(1.0f, 0.0f, 0.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(1.0f, 1.0f, 0.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(1.0f, 1.0f, 1.0f);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(1.0f, 0.0f, 1.0f);

		glNormal3f(-1.0f, 0.0f, 0.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(0.0f, 0.0f, 0.0f);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(0.0f, 0.0f, 1.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(0.0f, 1.0f, 1.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(0.0f, 1.0f, 0.0f);
		glEnd();
		glPopMatrix();
	glEndList();
	gluDeleteQuadric(quadObj);
}

void drawPlane(GLfloat a, GLfloat b, GLfloat c, GLfloat d, GLfloat fExtent, GLfloat fStep)
{
	GLfloat iLine;
	GLfloat angle;

//	glPushMatrix();
//	angle = -acos(b);
//	b = b - 1;
//	glRotatef(angle, a, b, c);
	glBegin(GL_LINES);
	for(iLine = -fExtent; iLine <= fExtent; iLine += fStep)
	{
		glVertex3f(iLine, d, fExtent);
		glVertex3f(iLine, d, -fExtent);
		glVertex3f(fExtent, d, iLine);
		glVertex3f(-fExtent, d, iLine);
	}
	glEnd();
//	glPopMatrix();
}

void updateCamera(POINT mousePos)
{
	static POINT	oldPos;
	POINT		mouseDelta;
	GLfloat		scale = 0.01f, vRight[3];
	GLfloat		matrix[16] = {	1.0f, 0.0f, 0.0f, 0.0f,
					0.0f, 1.0f, 0.0f, 0.0f, 
					0.0f, 0.0f, 1.0f, 0.0f, 
					0.0f, 0.0f, 0.0f, 1.0f	};

	mouseDelta.x = mousePos.x - oldPos.x;
	mouseDelta.y = mousePos.y - oldPos.y;
	oldPos = mousePos;

	camera.forward[0] = 0.0f;
	camera.forward[1] = 0.0f;
	camera.forward[2] = 1.0f;

	camera.up[0] = 0.0f;
	camera.up[1] = 1.0f;
	camera.up[2] = 0.0f;

	rotateVector(mousePos.x * scale, camera.forward, 0.0f, 1.0f, 0.0f);

	rotateVector(mousePos.y * scale, camera.forward, 1.0f, 0.0f, 0.0f);
	rotateVector(mousePos.y * scale, camera.up, 1.0f, 0.0f, 0.0f);
	crossProduct(vRight, camera.up, camera.forward);

	if (GetAsyncKeyState(VK_UP))
	{
		camera.pos[0] += camera.forward[0] * 2.0f;
		camera.pos[1] += camera.forward[1] * 2.0f;
		camera.pos[2] += camera.forward[2] * 2.0f;
//		camera.pos[2] -= 1.0;
	}
	if (GetAsyncKeyState(VK_DOWN))
	{
		camera.pos[0] -= camera.forward[0] * 2.0f;
		camera.pos[1] -= camera.forward[1] * 2.0f;
		camera.pos[2] -= camera.forward[2] * 2.0f;
//		camera.pos[2] += 1.0;
	}
	if (GetAsyncKeyState(VK_LEFT))
	{
		camera.pos[0] -= vRight[0] * 2.0f;
		camera.pos[1] -= vRight[1] * 2.0f;
		camera.pos[2] -= vRight[2] * 2.0f;
//		camera.pos[0] -= 1.0;
	}
	if (GetAsyncKeyState(VK_RIGHT))
	{
		camera.pos[0] += vRight[0] * 2.0f;
		camera.pos[1] += vRight[1] * 2.0f;
		camera.pos[2] += vRight[2] * 2.0f;
//		camera.pos[0] += 1.0;
	}
	if (GetAsyncKeyState(VK_RETURN))
	{
		camera.pos[0] += camera.up[0] * 2.0f;
		camera.pos[1] += camera.up[1] * 2.0f;
		camera.pos[2] += camera.up[2] * 2.0f;
//		camera.pos[1] += 1.0;
	}
	if (GetAsyncKeyState(VK_SHIFT))
	{
		camera.pos[0] -= camera.up[0] * 2.0f;
		camera.pos[1] -= camera.up[1] * 2.0f;
		camera.pos[2] -= camera.up[2] * 2.0f;
//		camera.pos[1] -= 1.0;

	}

	matrix[0]  = vRight[0];
	matrix[1]  = vRight[1];
	matrix[2]  = vRight[2];

	matrix[4]  = camera.up[0];
	matrix[5]  = camera.up[1];
	matrix[6]  = camera.up[2];

	matrix[8]  = camera.forward[0];
	matrix[9]  = camera.forward[1];
	matrix[10] = camera.forward[2];

	matrix[12] = -camera.pos[0];
	matrix[13] = -camera.pos[1];
	matrix[14] = -camera.pos[2];
	glMultMatrixf(matrix);
}

void crossProduct(GLfloat *vResult, GLfloat *VecA, GLfloat *VecB)
{
	vResult[0] = VecA[1] * VecB[2] - VecA[2] * VecB[1];
	vResult[1] = VecA[2] * VecB[0] - VecA[0] * VecB[2];
	vResult[2] = VecA[0] * VecB[1] - VecA[1] * VecB[0];
}

void drawText(const CHAR *string, GLfloat x, GLfloat y)
{
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
		glLoadIdentity();
		glOrtho(0.0f, 1.0f, 0.0f, 1.0f, 0.0f, 0.1f);
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
			glLoadIdentity();
			glRasterPos2f(x, y);
			glListBase(1000);
			glCallLists(lstrlen(string), GL_UNSIGNED_BYTE, string);	
		glPopMatrix();
		glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
}

void rotateVectorEuler(GLfloat *vec, GLfloat alpha, GLfloat beta, GLfloat gamma)
{
	GLfloat old[3], matrix[9], sinAlpha, sinBeta, sinGamma, cosAlpha, cosBeta, cosGamma;

	sinAlpha = sin(alpha);
	cosAlpha = cos(alpha);
	sinBeta = sin(beta);
	cosBeta = cos(beta);
	sinGamma = sin(gamma);
	cosGamma = cos(gamma);

	matrix[0] = cosAlpha * cosGamma - sinAlpha * cosBeta * sinGamma;
	matrix[1] = -sinAlpha * cosGamma - cosAlpha * cosBeta * sinGamma;
	matrix[2] = sinBeta * sinGamma;

	matrix[3] = cosAlpha * sinGamma + sinAlpha * cosBeta * cosGamma;
	matrix[4] = -sinAlpha * sinGamma + cosAlpha * cosBeta * cosGamma;
	matrix[5] = -sinBeta * cosGamma;

	matrix[6] = sinAlpha * sinBeta;
	matrix[7] = cosAlpha * sinBeta;
	matrix[8] = cosBeta;

	old[0] = vec[0];
	old[1] = vec[1];
	old[2] = vec[2];

	vec[0] = old[0] * matrix[0] + old[1] * matrix[1] + old[2] * matrix[2];
	vec[1] = old[0] * matrix[3] + old[1] * matrix[4] + old[2] * matrix[5]; 
	vec[2] = old[0] * matrix[6] + old[1] * matrix[7] + old[2] * matrix[8];
}

void rotateVectorX(GLfloat rad, GLfloat *vec) //roll
{
	GLfloat	old[3],
		matrix[9] = {	1.0f, 0.0f, 0.0f,
				0.0f, 0.0f, 0.0f,
				0.0f, 0.0f, 0.0f	};

	matrix[4] = cos(rad);
	matrix[8] = matrix[4];
	matrix[7] = sin(rad);
	matrix[5] = -matrix[7];

	old[0] = vec[0];
	old[1] = vec[1];
	old[2] = vec[2];

	vec[0] = old[0] * matrix[0] + old[1] * matrix[1] + old[2] * matrix[2];
	vec[1] = old[0] * matrix[3] + old[1] * matrix[4] + old[2] * matrix[5]; 
	vec[2] = old[0] * matrix[6] + old[1] * matrix[7] + old[2] * matrix[8];
}

void rotateVectorY(GLfloat rad, GLfloat *vec) //pitch
{
	GLfloat	old[3],
		matrix[9] = {	0.0f, 0.0f, 0.0f,
				0.0f, 1.0f, 0.0f,
				0.0f, 0.0f, 0.0f	};

	matrix[0] = cos(rad);
	matrix[8] = matrix[0];
	matrix[2] = sin(rad);
	matrix[6] = -matrix[2];

	old[0] = vec[0];
	old[1] = vec[1];
	old[2] = vec[2];

	vec[0] = old[0] * matrix[0] + old[1] * matrix[1] + old[2] * matrix[2];
	vec[1] = old[0] * matrix[3] + old[1] * matrix[4] + old[2] * matrix[5]; 
	vec[2] = old[0] * matrix[6] + old[1] * matrix[7] + old[2] * matrix[8];
}

void rotateVectorZ(GLfloat rad, GLfloat *vec) //yaw
{
	GLfloat	old[3],
		matrix[9] = {	0.0f, 0.0f, 0.0f,
				0.0f, 0.0f, 0.0f,
				0.0f, 0.0f, 1.0f	};

	matrix[0] = cos(rad);
	matrix[4] = matrix[0];
	matrix[3] = sin(rad);
	matrix[1] = -matrix[3];

	old[0] = vec[0];
	old[1] = vec[1];
	old[2] = vec[2];

	vec[0] = old[0] * matrix[0] + old[1] * matrix[1] + old[2] * matrix[2];
	vec[1] = old[0] * matrix[3] + old[1] * matrix[4] + old[2] * matrix[5]; 
	vec[2] = old[0] * matrix[6] + old[1] * matrix[7] + old[2] * matrix[8];
}


void rotateVector(GLfloat rad, GLfloat *vec, GLfloat x, GLfloat y, GLfloat z)
{
	GLfloat old[3], matrix[8], sinVal, cosVal, minusVal;

	sinVal = sin(rad);
	cosVal = cos(rad);
	minusVal = 1 - cosVal;

	old[0] = vec[0];
	old[1] = vec[1];
	old[2] = vec[2];

	matrix[0] = cosVal + minusVal * x * x;
	matrix[1] = minusVal * x * y - sinVal * z;
	matrix[2] = minusVal * x * z + sinVal * y;

	matrix[3] = minusVal * y * x + sinVal * z;
	matrix[4] = cosVal + minusVal * y * y;
	matrix[5] = minusVal * y * z - sinVal * x;

	matrix[6] = minusVal * z * x - sinVal * y;
	matrix[7] = minusVal * z * y + sinVal * x;
	matrix[8] = cosVal + minusVal * z * z;

	vec[0] = old[0] * matrix[0] + old[1] * matrix[1] + old[2] * matrix[2];
	vec[1] = old[0] * matrix[3] + old[1] * matrix[4] + old[2] * matrix[5]; 
	vec[2] = old[0] * matrix[6] + old[1] * matrix[7] + old[2] * matrix[8];
}